var	editors = [],

	mceGetContent = function(mark) {
		// First, clear any selection
		var cursor = editors[0].getCursor();
		editors[0].setCursor(cursor.line, cursor.ch);
		editors[0].replaceSelection(mark);
		return (editors[0].getValue());
	};

const REQUIRE_CONFIG = JSON.parse(document.body.querySelector('script[data-require-config]').getAttribute('data-require-config'));

requirejs.config(REQUIRE_CONFIG);

requirejs([
		/* from easymde
		'./codemirror/tablist',
		'addon/selection/mark-selection',
		'addon/search/searchcursor',
		 * */

		'lib/codemirror',
		'addon/comment/continuecomment',
		'addon/comment/comment',
		'addon/display/fullscreen',
		'addon/edit/closebrackets',
		'addon/edit/matchbrackets',
		'addon/display/panel',
		'addon/display/placeholder',
		'addon/edit/closetag',
		'addon/edit/continuelist', // specific markdown
		'addon/edit/matchtags',
		'addon/edit/trailingspace',
		'addon/fold/brace-fold',
		'addon/fold/comment-fold',
		'addon/fold/foldgutter',
		'addon/fold/foldcode',
		'addon/fold/indent-fold',
		'addon/fold/markdown-fold',
		'addon/lint/lint',
		'addon/lint/javascript-lint',
		'addon/lint/json-lint',
		'addon/lint/css-lint',
		'addon/hint/show-hint',
		'addon/hint/xml-hint',
		'addon/hint/html-hint',
		'addon/hint/css-hint',
		'addon/hint/javascript-hint',
		'addon/mode/overlay', // required for using spell-checker
		'addon/search/search',
		'addon/search/jump-to-line',
		'mode/php/php',
		'mode/gfm/gfm',
		'keymap/vim',
		'keymap/sublime',
		'keymap/emacs'
	],
	function(CodeMirror) {

		'use strict';

		const CODEMIRROR_OPTIONS = JSON.parse(document.body.querySelector('script[data-cm-options]').getAttribute('data-cm-options'));

		function fullscreen(cm) {
			if(window.frameElement == null) {
				cm.setOption("fullScreen", !cm.getOption("fullScreen"));
			} else {
				if(window.mceFullscreen) {
					window.mceFullscreen();
				}
			}
		}

		function helpMe(editor) {
			const panel = document.body.querySelector('.cm-help-content');
			if(panel != null) {
				panel.classList.toggle('active');
			} else {
				console.log('Element with .cm-help-content not found');
			}
		}

		function saveDocument(editor) {
			storeHeights(null);
			editor.getTextArea().form.submit();
		}

		CodeMirror.defaults.matchBrackets = true;
		CodeMirror.defaults.autoCloseBrackets = true;
		CodeMirror.defaults.autoCloseTags = true;
		CodeMirror.defaults.indentUnit = 4;
		CodeMirror.defaults.lineWrapping = true;
		CodeMirror.defaults.foldGutter = true;
		CodeMirror.defaults.gutters = [
			'CodeMirror-linenumbers',
			'CodeMirror-foldgutter',
			'CodeMirror-lint-markers'
		];

		CodeMirror.defaults.allowDropFileTypes = [
			'text/plain',
			'text/javascript',
			'application/json',
			'text/css'
		];

		CodeMirror.defaults.extraKeys = {
			'Alt-Z'	: function(cm) { cm.foldCode(cm.getCursor()); },
			Esc		: function(cm) { if (cm.getOption("fullScreen")) cm.setOption("fullScreen", false); },
			F11		: 'fullscreen',
			'Ctrl-F1': 'helpMe',
			'Ctrl-Space': 'autocomplete',
		};

		for(var option in CODEMIRROR_OPTIONS) {
			if(option != 'emmet') {
				CodeMirror.defaults[option] = CODEMIRROR_OPTIONS[option];
			}
		}

		CodeMirror.commands.fullscreen = fullscreen;
		CodeMirror.commands.helpMe = helpMe;
		CodeMirror.commands.save = saveDocument;

		if('emmet' in CODEMIRROR_OPTIONS) {
			console.log('Emmet required');
		}

		/* ************* Events callbacks ***************** */
		function storeHeights(event) {
			const statusBars = document.body.querySelectorAll('div.statusbar');
			const statusBarsCount = statusBars.length;
			if(statusBarsCount > 0) {
				for(var i=0; i<statusBarsCount; i++) {
					if(typeof statusBars[i].editor != undefined) {
						const editor = statusBars[i].editor;
						const textArea = editor.getTextArea();
						if(textArea != null) {
							const key = getKey(textArea);
							const height = editor.getWrapperElement().style.height;
							if(height.length > 0) {
								sessionStorage.setItem(key, height);
							} else {
								sessionStorage.removeItem(key);
							}
							const cursor = editor.getCursor();
							sessionStorage.setItem(key+'-line', cursor.line);
							sessionStorage.setItem(key+'-ch', cursor.ch);
						}
						if(editor.getOption('fullScreen') === true) {
							sessionStorage.setItem('CM-fullScreen', textArea.name);
						} else {
							sessionStorage.removeItem('CM-fullScreen');
						}
					}
				}
			}
		}

		function getKey(textarea) {
			return 'CM-' + window.location.pathname.replace(/^.*\/(\w+)\.php$/, '$1') + '-' + textarea.id;
		}

		function helpDisplay(event) {
			event.preventDefault();
			const panel = document.querySelector('.cm-help-content');
			if(panel != null) {
				panel.classList.toggle('active');
			}
		}

		function statusbarClick(event) {
			if(event.target.tagName == 'SPAN') {
				if(event.target.hasAttribute('data-command')) {
					event.preventDefault();
					this.editor.execCommand(event.target.getAttribute('data-command'));
				}
			}
		}

		/* ------------ for markdown mode ----------- */

		function _toggleBlock(editor, type, start_chars, end_chars) {
			alert('_toggleBlock not implemented');
		}

		function _toggleHeading(cm, direction, size) {
		}

		function _toggleLine(cm, name) {

		}

		function toggleHeadingSmaller(editor) {
		    _toggleHeading(editor.codemirror, 'smaller');
		}

		function toggleHeadingBigger(editor) {
		    _toggleHeading(editor.codemirror, 'bigger');
		}

		function toggleHeading1(editor) {
		    _toggleHeading(editor.codemirror, undefined, 1);
		}

		function toggleHeading2(editor) {
		    _toggleHeading(editor.codemirror, undefined, 2);
		}

		function toggleHeading3(editor) {
		    _toggleHeading(editor.codemirror, undefined, 3);
		}

		function toggleUnorderedList(editor) {
		    _toggleLine(editor.codemirror, 'unordered-list');
		}

		function toggleOrderedList(editor) {
		    _toggleLine(editor.codemirror, 'ordered-list');
		}

		function toggleBold(editor) {
		    _toggleBlock(editor, 'bold', editor.options.blockStyles.bold);
		}

		function toggleItalic(editor) {
		    _toggleBlock(editor, 'italic', editor.options.blockStyles.italic);
		}

		function toggleStrikethrough(editor) {
		    _toggleBlock(editor, 'strikethrough', '~~');
		}

		function toggleBlockquote(editor) {
		    _toggleLine(editor.codemirror, 'quote');
		}

		function toggleHeadingSmaller(editor) {
		    _toggleHeading(editor.codemirror, 'smaller');
		}

		function toggleHeadingBigger(editor) {
		    _toggleHeading(editor.codemirror, 'bigger');
		}

		function toggleHeading1(editor) {
		    _toggleHeading(editor.codemirror, undefined, 1);
		}

		function toggleHeading2(editor) {
		    _toggleHeading(editor.codemirror, undefined, 2);
		}

		function toggleHeading3(editor) {
		    _toggleHeading(editor.codemirror, undefined, 3);
		}


		function toggleUnorderedList(editor) {
		    _toggleLine(editor.codemirror, 'unordered-list');
		}

		function toggleOrderedList(editor) {
		    _toggleLine(editor.codemirror, 'ordered-list');
		}

		function createToolbar(editor) {
				const toolbarBuiltInButtons = {
					'bold': {
						action: toggleBold,
						title: 'Bold',
						default: true,
					},
					'italic': {
						action: toggleItalic,
						title: 'Italic',
						default: true,
					},
					'strikethrough': {
						action: toggleStrikethrough,
						icon: 'strike',
						title: 'Strikethrough',
					},
					'heading': {
						action: toggleHeadingSmaller,
						icon: 'header',
						title: 'Heading',
						default: true,
					},
					'heading-smaller': {
						action: toggleHeadingSmaller,
						icon: 'header',
						className: 'heading-smaller',
						title: 'Smaller Heading',
					},
					'heading-bigger': {
						action: toggleHeadingBigger,
						icon: 'header',
						className: 'heading-bigger',
						title: 'Bigger Heading',
					},
					'heading-1': {
						action: toggleHeading1,
						icon: 'header',
						className: 'heading-1',
						title: 'Big Heading',
					},
					'heading-2': {
						action: toggleHeading2,
						icon: 'header',
						className: 'heading-2',
						title: 'Medium Heading',
					},
					'heading-3': {
						action: toggleHeading3,
						icon: 'header',
						className: 'heading-3',
						title: 'Small Heading',
					},
					'separator-1': {
						name: 'separator-1',
					},
					'code': {
						action: toggleCodeBlock,
						icon: 'code',
						title: 'Code',
					},
					'quote': {
						action: toggleBlockquote,
						icon: 'quote-left',
						title: 'Quote',
						default: true,
					},
					'unordered-list': {
						action: toggleUnorderedList,
						title: 'Generic List',
						default: true,
					},
					'ordered-list': {
						action: toggleOrderedList,
						icon: 'list-numbered',
						title: 'Numbered List',
						default: true,
					},
					'clean-block': {
						action: cleanBlock,
						icon: 'eraser',
						title: 'Clean block',
					},
					'separator-2': {
						name: 'separator-2',
					},
					'link': {
						action: drawLink,
						title: 'Create Link',
						default: true,
					},
					'picture': {
						action: drawImage,
						title: 'Insert Image',
						default: true,
					},
					'upload-picture': {
						action: drawUploadedImage,
						icon: 'picture',
						title: 'Import an image',
					},
					'table': {
						action: drawTable,
						title: 'Insert Table',
					},
					'horizontal-rule': {
						action: drawHorizontalRule,
						icon: 'minus',
						title: 'Insert Horizontal Line',
					},
					'separator-3': {
						name: 'separator-3',
					},
					'preview': {
						action: togglePreview,
						icon: 'eye',
						noDisable: true,
						title: 'Toggle Preview',
						default: true,
					},
					'side-by-side': {
						icon: 'columns',
						noDisable: true,
						noMobile: true,
						title: 'Toggle Side by Side',
						default: true,
					},
					'fullscreen': {
						action: toggleFullScreen,
						icon: 'resize-full-alt',
						noDisable: true,
						noMobile: true,
						title: 'Toggle Fullscreen',
						default: true,
					},
					'save' : {
						action: saveDoc,
						icon: 'floppy',
						title: 'Save',
					},
					'separator-4': {
						name: 'separator-4',
					},
					'guide': {
						action: 'https://www.markdownguide.org/basic-syntax/',
						icon: 'help-circled',
						noDisable: true,
						title: 'Markdown Guide',
						default: true,
					},
					'guide1': {
						action: 'https://github.github.com/gfm/',
						icon: 'help-circled',
						noDisable: true,
						title: 'GitHub Flavored Markdown',
						default: true,
					},
					'separator-5': {
						name: 'separator-5',
					},
					'undo': {
						action: undo,
						icon: 'ccw',
						noDisable: true,
						title: 'Undo',
					},
					'redo': {
						action: redo,
						icon: 'cw',
						noDisable: true,
						title: 'Redo',
					},
				};

			const toolbar = document.createElement('div');
			toolbar.classList.add('toolbar');

			for(i in toolbarBuiltInButtons) {
				const btn = document.createElement('BUTTON');
				btn.type = 'button';
				btn.className = 'icon-' + (('icon' in toolbarBuiltInButtons[i]) ? toolbarBuiltInButtons[i].icon : i);
				if('title' in toolbarBuiltInButtons[i]) {
					btn.title = toolbarBuiltInButtons[i].title;
				}
			}

			toolbar.addEventListener('click', toolbarClick);

			toolbar.editor = editor;

			return toolbar;
		}

		function createStatusbar(editor) {
			const statusbar = document.createElement('DIV');
			// statusbar.id = 'codemirror-' + elmt.id;
			statusbar.classList.add('statusbar');
			var innerHTML =
				'<div>' +
					'<span>Codemirror plugin</span>' +
					'<span data-command="helpMe">Help: Ctrl-F1</span>' +
					'<span data-command="save">Save: Ctrl-S</span>' +
					'<span data-command="fullscreen">Fullscreen: F11</span>';
			if('keyMap' in CODEMIRROR_OPTIONS) {
				innerHTML += '<span>' + CODEMIRROR_OPTIONS.keyMap + '</span>'
			}
			innerHTML +=
					'<span>Codemirror - version ' + CodeMirror.version + '</span>' +
				'</div>' +
				'<div class="spacer"></div>' +
					'<div><span class="posCursor">&nbsp;</span></div>';
			statusbar.innerHTML = innerHTML;

			statusbar.addEventListener('click', statusbarClick);

			statusbar.editor = editor;

			return statusbar;
		}

		const helpPanel = document.body.querySelector('.cm-help-content');
		if(helpPanel != null) {
			helpPanel.addEventListener('click', function(event) {
				event.preventDefault();
				this.classList.toggle('active');
			});
		}

		if(top.tinymce != null) {
			// Intégration dans Tinymce
			var ed = CodeMirror(document.body);
			ed.setOption('fullScreen', true);

			var mceActiveEditor = top.tinymce.activeEditor;
			var content = mceActiveEditor.getContent();
			// supprime la marque du cursor dans Tinymce
			mceActiveEditor.execCommand('Undo');
			ed.setValue(content);
			// remove the mark and place the cursor in Codemirror
			var mceParams = mceActiveEditor.windowManager.getParams();
			var cursor = ed.getSearchCursor(mceParams.mark);
			if(cursor.find()) {
				cursor.replace('');
				ed.setCursor(cursor.from());
			}
			ed.focus();

			editors.push(ed);
		} else {
			const TEXTAREAS = 'content chapo backend frontend sandbox'.split(' ').map(function(item) {
				return item.replace(/^(\w+)$/, 'textarea[name="$1"]')
			});
			const contents = document.body.querySelectorAll(TEXTAREAS);
			const contentsCount = contents.length;
			if(contentsCount > 0) {
				for(var i=0; i<contentsCount; i++) {
					const elmt = contents[i];
					var ed = CodeMirror.fromTextArea(elmt);
					const statusbar = createStatusbar(ed);
					ed.addPanel(
						statusbar,
						{ position: 'bottom'}
					);
					editors.push(ed);
					ed.on(
						'cursorActivity',
						function(editor) {
							if(editor.hasFocus()) {
								const cursor = editor.getCursor();
								const container = editor.getWrapperElement();
								const panel = container.parentElement.querySelector('.statusbar .posCursor');
								panel.innerHTML = (cursor.line + 1) + ':' + (cursor.ch + 1);
							}
							if('emmet' in CODEMIRROR_OPTIONS) {
								ed.on('cursorActivity', emmetCursorActivity);
							}
						}
					);

					const key = getKey(elmt);
					const height = sessionStorage.getItem(key);
					if(height != null) {
						ed.getWrapperElement().style.height = parseInt(height);
					}
					const line = sessionStorage.getItem(key + '-line');
					const ch = sessionStorage.getItem(key + '-ch');
					if(line != null && ch != null) {
						ed.setCursor(parseInt(line), parseInt(ch));
					}
					const fullWrapper = sessionStorage.getItem('CM-fullScreen');
					if(fullWrapper != null && fullWrapper == elmt.name) {
						ed.setOption('fullScreen', true);
					}
					if(elmt.form != null) {
						elmt.form.addEventListener('submit', storeHeights);
					}
				}
				document.body.classList.add('codemirror-plugin');
			}

			// gestion des onglets
			if(document.body.id == 'parametres_plugincss') {
				const PATTERN = 'fieldset div.grid > div > label';
				var ACTIVE = 'active';

				const tabs = document.body.querySelectorAll(PATTERN);
				const tabsCount = tabs.length;
				if(tabsCount > 0) {

					function activeTab(event) {
						event.preventDefault();
						const actives = document.querySelectorAll('fieldset div.grid > div > label.active');
						const activesCount = actives.length;
						if(activesCount > 0) {
							for(var i=0; i<activesCount; i++) {
								actives[i].classList.remove(ACTIVE);
							}
						}
						event.target.classList.add(ACTIVE);
					}

					for(var i=0; i<tabsCount; i++) {
						tabs[i].addEventListener('click', activeTab);
						if(i == 0) {
							tabs[0].classList.add(ACTIVE);
						}
					}
				}
			}
		}
	}
);
