<?php
if(!defined('PLX_ROOT')) { exit; }

// https://codemirror.net
// https://github.com/emmetio/codemirror-plugin
// https://github.com/erusev/parsedown

class kzCodemirror extends plxPlugin {

	const CM_LIBRARY_CDN = 'https://cdnjs.cloudflare.com/ajax/libs/codemirror/';
	const API_CDNJS = 'https://api.cdnjs.com/libraries/codemirror';

	const CM_VERSION = '6.65.7'; // only requested for cdn : https://cdnjs.com/libraries/codemirror

	const CM_ADDONS_STYLESHEETS =
		'addon/dialog/dialog addon/display/fullscreen '.
		'addon/fold/foldgutter addon/hint/show-hint addon/lint/lint '.
		'addon/hint/show-hint '.
		'addon/scroll/simplescrollbars addon/search/matchesonscrollbar';
	const HOOKS = array(
		'AdminTopEndHead',
		'AdminFootEndBody',
	);
	const PROFILS = array(
		PROFIL_ADMIN		=> 1,
		PROFIL_MANAGER		=> 2,
		PROFIL_MODERATOR	=> 4,
		PROFIL_EDITOR		=> 8,
		PROFIL_WRITER		=> 16
	);

	public $mode = false;
	public $local_path = false; // Si faux, Codemirror est téléchargé depuis Internet
	public $cm_url = false; // sans slash final !
	public $config = false; // Set by core/admin/parametres_plugin.php?p=kzCodemirror - See AdminFootEndBody()
	public $disabled = false; // can change by core/admin/parametres_plugin.php?p=kzCodemirror
	public $plx_plugin_root = PLX_PLUGINS . __CLASS__ . '/'; // A modifier si le plugin est appelé par TinyMCE

	public function __construct($default_lang) {
		parent::__construct($default_lang);

		if(defined('PLX_ADMIN')) {
			# droits pour accéder à la page config.php du plugin
			parent::setConfigProfil(PROFIL_ADMIN);

			$scriptname = basename(strtolower($_SERVER['SCRIPT_NAME']),'.php');
			if($scriptname == 'user') {
				$scriptname = 'profil';
			}

			if(
				(
					($scriptname == 'parametres_plugin') and
					!empty($_REQUEST['p']) and
					($_REQUEST['p'] == __CLASS__)
				) or
				(
					!empty($this->getParam($scriptname)) and
					(
						($scriptname != 'article') or
						$this->__article_enabled()
					)
				)
			) {
				$this->mode = $scriptname;
			} else {
				return false;
			}
		} elseif(defined('TINYMCE_PLUGIN')) {
			$this->plx_plugin_root = '../';
		} else {
			return false;
		}

		foreach(self::HOOKS as $hook) {
			parent::addHook($hook, $hook);
		}

		self::__set_links();
	}

	public function saveParams() {
		parent::saveParams();
		if(empty($this->local_path) xor !empty($this->getParam('cdn'))) {
			$this->__set_links();
			$this->onActivate(true);
		}
	}

	private function __article_enabled() {
		return !empty($this->getParam('article') & self::PROFILS[$_SESSION['profil']]);
	}

	private function __set_links() {
		if(empty($this->getParam('cdn'))) {
			$filenames = glob(__DIR__ . '/codemirror-*/lib/codemirror.js');
			if(!empty($filenames)) {
				$folder = preg_replace(
					'@.*/(codemirror-[^/]*)/lib/codemirror.js@', "$1",
					$filenames[count($filenames)  - 1]
				);
				$this->local_path = __DIR__ . '/' . $folder; // sans slash final !
				if(!empty($this->plx_plugin_root)) {
					$this->cm_url =	$this->plx_plugin_root . $folder;
				}
				return;
			}
		}
		$this->local_path = false;
		$this->cm_url = $this::CM_LIBRARY_CDN . $this::CM_VERSION;
	}

	/**
	 * Récupère l'Url absolue pour le dossier du plugin.
	 * */
	public function get_context() {
		$this->plx_plugin_root = PLX_PLUGINS . __CLASS__ . '/';
		$this->__set_links();
	}

	/**
	 * Compile les feuilles de styles des addons de Codemirror lorsque le plugin est activé, hormis les feuilles des themes.
	 *
	 * Called by PlxPlugins::saveConfig() or $this::saveParams().
	 * */
	public function OnActivate($saveParams=false) {
		$banner = '/* ---------- codemirror.css ----------- */'  . PHP_EOL;

		if(!$saveParams) { $this->__set_links(); }

		ob_start();
		echo '/* ---- Don\'t change manually - build on ' . date('Y-m-d H:i') . ' ---- */' . PHP_EOL;

		if(empty($this->getParam('cdn'))) {
			$folder = substr(preg_replace('@^' . __DIR__ .' @', '', $this->local_path), 1);
			echo '/* Reading from ' . $folder . ' folder */' . PHP_EOL;
			echo $banner;
			readfile($this->local_path . '/lib/codemirror.css');
			readfile(__DIR__ . '/css/admin.css');

			foreach(explode(' ', $this::CM_ADDONS_STYLESHEETS) as $sheet) {
				echo '/* ---' . $sheet . '.css --- */';
				readfile($this->local_path . "/$sheet.css");
			}
		} else {
			$ch = curl_init();
			curl_setopt_array($ch, array(
				CURLOPT_FOLLOWLOCATION	=> true,
				CURLOPT_RETURNTRANSFER	=> false,
				CURLOPT_USERAGENT		=> 'curl/'.curl_version()['version'],
				CURLOPT_HEADER			=> false
			));
			$base_url = $this::CM_LIBRARY_CDN.$this::CM_VERSION.'/';
			echo $banner;
			echo '/* Download from' . $base_url . ' */' . PHP_EOL;
			curl_setopt($ch, CURLOPT_URL, $base_url . 'codemirror.css');
			curl_exec($ch);
			readfile(__DIR__.'/css/admin.css');

			foreach(explode(' ', $this::CM_ADDONS_STYLESHEETS) as $sheet) {
				curl_setopt($ch, CURLOPT_URL, $base_url."$sheet.css");
				echo '/* --- ' . $sheet . '.css --- */' . PHP_EOL;
				curl_exec($ch);
			}
			curl_close($ch);
		}

		// Pas de feuilles de style pour les modes utilisés.

		global $plxAdmin;

		$filename = PLX_ROOT . PLX_CONFIG_PATH . 'plugins/' . __CLASS__ . '.admin.css';
		if(
			!plxUtils::write(ob_get_clean(), $filename) or
			!$plxAdmin->plxPlugins->cssCache('admin')) {
			plxMsg::Error(L_SAVE_FILE_ERROR);
		} elseif($saveParams) {
			# génération du cache css des plugins
			$plxAdmin->plxPlugins->cssCache('admin');
		}
	}

	/**
	 * Crée des listes déroulantes (<select>) des thémes et des raccourci-claviers pour le panneau de configuration.
	 * */
	public function print_select($name, $value, $ext='css') {
?>
		<select id="id_<?= $name; ?>" name="<?= $name; ?>">
			<option value=""><?php $this->lang('L_DEFAULT'); ?></option>
<?php
		if(!empty($this->local_path)) {
			foreach(glob("{$this->local_path}/".strtolower($name)."/*$ext") as $filename) {
				$buf = basename($filename, ".$ext");
				$caption = ucfirst($buf);
				$selected = ($buf == $value) ? ' selected' : '';
?>
			<option value="<?= $buf ?>"<?= $selected ?>><?= $caption ?></option>
<?php
			}
		} else {
			if(!empty($value)) {
				$caption = ucfirst($value);
?>
			<option value="<?= $value ?>" selected><?= $caption ?></option>
<?php			}
		}
?>
		</select>
<?php
	}

	/**
	 * Charge la feuille de style CSS nécessaire à Codemirror pour le thème sélectionné.
	 * */
	public function AdminTopEndHead() {
		if($this->disabled) {
			# Plugin disabled. Set by config.php
?>
	<link rel="stylesheet" type="text/css" href="<?= $this->plx_plugin_root ?>css/admin.css">
<?php
		}

		$theme = $this->getParam('theme');
		if(!empty($this->getParam('theme'))) {
			$href = $this->cm_url."/theme/$theme.css";
?>
	<link type="text/css" rel="<?php if($this->mode == 'parametres_plugin') echo 'alternate '; ?>stylesheet" title ="<?= $theme; ?>" href="<?= $href; ?>" />
<?php
		}
	}

	/**
	 * Précise à Codemirror le langage (mode) à utiliser.
	 * */
	private function __cm_mode() {
		switch($this->mode) {
			case 'parametres_plugin':
			case 'parametres_plugincss' :
				return 'css';
				break;
			case 'parametres_edittpl':
				global $tpl;
				return pathinfo($tpl, PATHINFO_EXTENSION);
				break;
			case 'article':
				if(empty($this->getParam('markdown'))) {
					return 'application/x-httpd-php';
					// return 'mixedMode';
				}
				// return 'markdown';

				return array(
					'name'	=> 'gfm',
					'tokenTypeOverrides' => array('emoji' => 'emoji'),
				);
				break;
			default:
				return 'application/x-httpd-php';
		}
	}

	/**
	 * Génére la liste des options de Codemirror en fonction du choix utilisateur.
	 * */
	private function __print_options() {
		$options = array(
			'mode'		=> $this->__cm_mode(),
			'extraKeys'	=> array('Enter' => 'newlineAndIndentContinueMarkdownList'), // for Markdown format
		);

		# checkbox inputs
		foreach(array('lineNumbers', 'lint', 'matchTags', 'emmet') as $option) {
			if(!empty($this->getParam($option))) {
				$options[$option] = true;
			}
		}

		foreach(array('keyMap', 'theme') as $param) {
			$value = $this->getParam($param);
			if(!empty($value)) {
				$options[$param] = $value;
			}
		}

		echo json_encode(
			$options,
			JSON_UNESCAPED_SLASHES + JSON_UNESCAPED_UNICODE + JSON_FORCE_OBJECT
		);
	}

	public function print_profils() {
		$this->aLang['L_ARTICLE']				= L_MENU_ARTICLES;
		$this->aLang['L_STATIQUE']				= L_MENU_STATICS;
		$this->aLang['L_COMMENT']				= defined('L_COMMENTS') ? L_COMMENTS : L_MENU_COMMENTS;
		$this->aLang['L_CATEGORIE']				= L_CATEGORIES;
		$this->aLang['L_PARAMETRES_PLUGINCSS']	= L_MENU_CONFIG_PLUGINS;
		$this->aLang['L_PARAMETRES_EDITTPL']	= L_THEMES;
?>
		<p>
			<span class="profils-label"><?php $this->lang('L_PROFILS'); ?></span>
<?php
		foreach(self::PROFILS as $profil=>$weigth) {
?>
		<span class="profil" title="<?php $this->lang('L_PROFIL_'.$profil)?>"><?php $this->lang('L_SHORT_PROFIL_'.$profil)?></span>
<?php
		}
?>
		</p>
<?php
	}

	public function print_checkboxes_profils($name, $param) {
		$value = intval($param);
		foreach(self::PROFILS as $profil=>$weight) {
			$checked = (!empty($value & $weight)) ? ' checked' : '';
?>
		<input class="profil" type="checkbox" name="<?= $name; ?>[]" value="<?= $weight; ?>"<?= $checked; ?> />
<?php
		}
	}

	/**
	 * Liste des paramètres pour la librairie requirejs.
	 * */
	private function __print_require_config() {
		$config = array(
			'baseUrl'		=> $this->cm_url,
			'waitSeconds'	=> 15,
			'paths'			=> array(
				'emmet'	=> $this->plx_plugin_root.'emmet/emmet-codemirror-plugin.js'
			),
			'shim'			=> array(
				'emmet'	=> array(
					'exports'	=> 'Emmet'
				)
			)
		);

		if(empty($this->local_path)) {
			// Les librairies sont téléchargées depuis Internet
			$config['paths'] =  array(
				'lib/codemirror' => 'codemirror'
			);
		}

		echo json_encode(
			$config,
			JSON_UNESCAPED_SLASHES + JSON_UNESCAPED_UNICODE + JSON_FORCE_OBJECT
		);
	}

	/* ------------------------ Hooks start here ---------------------------- */

	public function plxMotorConstruct() {
		echo '<?php $this->plxPlugins->aPlugins["' . __CLASS__ . '"]->get_context(); ?>';
	}

	/**
	 * Charge les scripts Javascript nécessaires pour les plugins (addons) et modes de Codemirror.
	 * Définit la configuration de ce dernier. Détecte si le bundle main.js pour Codemirror existe.
	 * */
	public function AdminFootEndBody() {
		if($this->disabled and $this->config) {
			return;
		}

		$data_main = $this->plx_plugin_root;
		// check if library is optimized with r.js
		$data_main .= (empty($this->getParam('cdn')) and file_exists(__DIR__ . '/main.js')) ? 'main.js' : 'app.js';
?>
	<div class="cm-help-content">
<?php readfile( __DIR__ . '/lang/' . $this->default_lang . '-help-content.html'); ?>
	</div>
	<script type="text/javascript" src="<?= $this->plx_plugin_root; ?>require.js"
		data-main="<?= $data_main; ?>"
		data-cm-options='<?php $this->__print_options(); ?>'
		data-require-config='<?= $this->__print_require_config(); ?>'
	></script>
<?php
		if(
			// ($this->mode != 'parametres_plugin') and
			!empty($this->getParam('lint'))
		) {
?>
	<script src="<?= $this->plx_plugin_root; ?>csslint.js" defer></script>
<?php
		}

		if($this->config) {
			# configuration du plugin - démo pour les thèmes
?>
<script>
	(function() {
		'use strict';

		const THEMES_SELECT = document.getElementById('id_theme');
		if(THEMES_SELECT == null) {
			return;
		}

		const THEMES_ROOT = '<?= $this->cm_url; ?>/theme/';
		var themes = [
<?php
	$theme = $this->getParam('theme');
	if(!empty($theme)) {
?>
			'<?= $theme ?>',
<?php
	}
?>
		];

		function activateTheme(theme) {
			for(const sh of document.styleSheets) {
				const matches = sh.href.match(/\/<?= __CLASS__ ?>\/codemirror-\d+.*\/theme\/(.*)\.css$/);
				if(matches) {
					sh.disabled = (theme.length == 0 || matches[1] != theme);
				}
			}

			if(typeof editors != 'undefined') {
				editors.forEach(function(ed) {
					ed.setOption('theme', theme);
				});
			}
		}

		function onLinkLoaded() {
			const theme = this.title;
			console.log('<?= __CLASS__ ?> plugin : Theme ' + theme + ' loaded');
			themes.push(theme);
			activateTheme(theme);
		}

		THEMES_SELECT.addEventListener('change', function(event) {
			const theme = event.target.value;
			if(theme.length > 0 && themes.indexOf(theme) < 0) {
				const stylesheet = document.createElement('LINK');
				stylesheet.type = 'text/css';
				stylesheet.rel = 'alternate stylesheet';
				stylesheet.title = theme;
				stylesheet.href = THEMES_ROOT + theme + '.css';
				stylesheet.onload = onLinkLoaded;
				document.head.appendChild(stylesheet);
			} else {
				if(theme.length > 0) {
					console.log('<?= __CLASS__ ?> plugin : Theme : ' + theme + ' - ' + themes.length + ' themes already uploaded : ' + themes);
				}
				activateTheme(theme);
			}
		});

		window.addEventListener('load', function(ev) {
			if(themes.length > 0) {
				activateTheme(themes[0]);
			}
		});
	})();
</script>
<?php
		}
	}

}
